-- Jarcast Amalfi Functions
-- Author: Jarcast
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
WARN_NOT_SHARED = false; include( "SaveUtils" ); MY_MOD_NAME = "Jarcast_Amalfi";

include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
include("AdditionalAchievementsUtility.lua");
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_IsPlayerAtWarWithAnyone(playerID, bIncludeMinors)
    local pPlayer = Players[playerID]
    local iTeam = pPlayer:GetTeam()
    local pTeam = Teams[iTeam]
    local bIsAtWar = false
    for i = 0, GameDefines.MAX_CIV_PLAYERS - 1 do -- Iterate through everyone
        if i ~= playerID then -- If this isn't the Player, since you can't be at war with yourself...
            local pOtherPlayer = Players[i]
            if pOtherPlayer:IsAlive() then -- Can't be at war with the dead :p
                if not pOtherPlayer:IsBarbarian() then -- You're always at war with Barbs so ignore them
                    if (bIncludeMinors) or ((not bIncludeMinors) and (not pOtherPlayer:IsMinorCiv())) then -- If you've said to include City-States, then everyone will qualify; if you've said to not include them, then make sure pOtherPlayer isn't a City-State
                        local iOtherTeam = pOtherPlayer:GetTeam()
                        if pTeam:IsAtWar(iOtherTeam) then -- The crucial Method
                            bIsAtWar = true
                            break -- We've found someone we're at war with, so we don't need to check for any longer
                        end
                    end
                end
            end
        end
    end
    return bIsAtWar -- Returns either true or false to wherever you called the function
end
-------------------------------------------------------------------------------------------------------------------------
function Civ1HasOpenBorderFromCiv2(iPlayer1, iPlayer2)
	if Teams[Players[iPlayer2]:GetTeam()]:IsAllowsOpenBordersToTeam( Players[iPlayer1]:GetTeam() ) then
        return true
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
-------------------------------------------------------------------------------------------------------------------------
-- base converter
local floor,insert = math.floor, table.insert
function basen(n,b)
    n = floor(n)
    if not b or b == 10 then return tostring(n) end
    local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    local t = {}
    local sign = ""
    if n < 0 then
        sign = "-"
    n = -n
    end
    repeat
        local d = (n % b) + 1
        n = floor(n / b)
        insert(t, 1, digits:sub(d,d))
    until n == 0
    return sign .. table.concat(t,"")
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
local DummyFondacoClassID = GameInfo.BuildingClasses.BUILDINGCLASS_JAR_DUMMY_FONDACO.ID

function Jar_PlayerHasFondaco(pPlayer)		
	local HasFondaco = pPlayer:GetBuildingClassCount(DummyFondacoClassID) > 0
	return HasFondaco
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_CountAlivePlayers()
	local plcount = 0
	for playerID = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pPlayer = Players[playerID];
		if pPlayer:IsAlive() and not pPlayer:IsBarbarian() then
			plcount = plcount +1
		end
	end
	return plcount
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_CountFondaci()
	local fondacount = 0
	for playerID = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pPlayer = Players[playerID];
		if pPlayer:IsAlive() and Jar_PlayerHasFondaco(pPlayer) then
			fondacount = fondacount +1
		end
	end
	return fondacount
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_PlotHasTradeRoute(pPlot, pPlayer)
	return (#pPlayer:GetInternationalTradeRoutePlotToolTip(pPlot) > 0)
end
--==========================================================================================================================
-- GAME DEFINES
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local CSDecayXTurn = GameDefines.MINOR_FRIENDSHIP_DROP_PER_TURN /(-100)
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iCiv = GameInfoTypes["CIVILIZATION_JAR_AMALFI"]
local traitAmalfiID = GameInfoTypes["TRAIT_JAR_AMALFI"]
local g_IsTraitActive = IsTraitActive(traitAmalfiID)
local DummyFreePolicy = GameInfoTypes["BUILDING_JAR_DUMMY_FREE_POLICY"]
local DummyUACoastalMerchant = GameInfoTypes["BUILDING_JAR_DUMMY_AMALFIUA"]
local iComite = GameInfoTypes["UNIT_JAR_COMITE"]
local missionComiteBuildFondaco = GameInfoTypes["MISSION_JAR_COMITE_FONDACO"]
local iFondaco = GameInfoTypes["IMPROVEMENT_JAR_FONDACO"]
local DummyFondaco = GameInfoTypes["BUILDING_JAR_DUMMY_FONDACO"]
local iBorgoCostiero = GameInfoTypes["IMPROVEMENT_JAR_SCARO"]
local BorgoSuppliesPromo = GameInfoTypes["PROMOTION_BORGO_SUPPLIES"] 
local DummyBorgoTRrange = GameInfoTypes["BUILDING_JAR_DUMMY_BORGO_TRRANGE"]
local DummyTRSlot = GameInfoTypes["BUILDING_JAR_DUMMY_FREE_TRROUTE_0"]
local DummyProdNaval = GameInfoTypes["BUILDING_JAR_DUMMY_NAVAL_PRODUC_0"]
local DummyTourism = GameInfoTypes["BUILDING_JAR_DUMMY_TOURISM"]
local eraIndustrialID 	 = GameInfoTypes["ERA_INDUSTRIAL"]
local iCoast = GameInfoTypes.TERRAIN_COAST
local iOcean = GameInfoTypes.TERRAIN_OCEAN
local CUSTOM_MISSION_NO_ACTION		 	= 0
local CUSTOM_MISSION_ACTION			 	= 1
local CUSTOM_MISSION_DONE            	= 2
local CUSTOM_MISSION_ACTION_AND_DONE 	= 3

local maxpower2 = 4
tTRslotDummies = {}
for power=0,maxpower2 do
	tTRslotDummies[power] = GameInfoTypes["BUILDING_JAR_DUMMY_FREE_TRROUTE_" .. tostring(power)]
end
local maxpower3 = 5
tShipProdDummies = {}
for power=0,maxpower3 do
	tShipProdDummies[power] = GameInfoTypes["BUILDING_JAR_DUMMY_NAVAL_PRODUC_" .. tostring(power)]
end
tMerchantCoastDummies = {}
for power=0,maxpower2 do
	tMerchantCoastDummies[power] = GameInfoTypes["BUILDING_JAR_DUMMY_AMALFIUA_" .. tostring(power)]
end
------------------------------------------------------------------------------------------------------------------------
-- QUICK MOD BALANCING PANEL
------------------------------------------------------------------------------------------------------------------------
-- Here are the changeable values that determine the strength of the effects described in the design.
-- UA
local UAfactor = 1/4
local UAFreePolicyCap = 4
local NavalUnitProdBoost = 15
local NavalUnitProdBoostMax = 50
-- UU
local AddTRfromXFondaci = 2
local CSInfluenceFondacoBase = math.floor(CSDecayXTurn)
-- UI
local BorgoTRrange = 5
local BorgoTRrangeMax = 30   
local BorgoShipHeal = -10
local BorgoShipDamage = 20
--========================================================================================================================
-- FUNCTIONS
--========================================================================================================================
-- UA: Prima Repubblica Marinara
--========================================================================================================================
tBuildingAmalfiUA = {}
for row in DB.Query("select ID From Buildings where IsDummy=0 AND (Water=1 OR SpecialistType ='SPECIALIST_MERCHANT')") do
	tBuildingAmalfiUA[row.ID] = true
end
function Jar_AddMerchantCoastDummy(pCity, integer)
	local binary = basen(integer,2)
    local numcifre = string.len(tostring(binary))
    for i=-numcifre,-1 do
        if string.len(tostring(binary)) == -i and binary~=0 then
            pCity:SetNumRealBuilding(tMerchantCoastDummies[-i-1], 1)
            binary = floor( binary - math.pow(10, -i-1) )
        else
            pCity:SetNumRealBuilding(tMerchantCoastDummies[-i-1], 0)
        end
    end
end
function Jar_AmalfiUACoastalMerchantBuildings(iPlayer)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsAlive() then return end
	if not HasTrait(pPlayer, traitAmalfiID) then return end
	for pCity in pPlayer:Cities() do
		local numUAbuilding= 0
		if pCity:IsHasBuilding(GameInfoTypes.BUILDING_PALACE) 		then numUAbuilding = numUAbuilding +1 end
		if pCity:IsHasBuilding(GameInfoTypes.BUILDING_CARAVANSARY) 	then numUAbuilding = numUAbuilding +1 end
		for k, v in pairs(tBuildingAmalfiUA) do
			if pCity:IsHasBuilding(k) then
				numUAbuilding = numUAbuilding +1
			end
		end
		if numUAbuilding>0 then
			Jar_AddMerchantCoastDummy(pCity, numUAbuilding)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiUACoastalMerchantBuildings) end;

function Jar_AddShipProdDummy(pCity, integer)
	local binary = basen(integer,2)
    local numcifre = string.len(tostring(binary))
    for i=-numcifre,-1 do
        if string.len(tostring(binary)) == -i and binary~=0 then
            pCity:SetNumRealBuilding(tShipProdDummies[-i-1], 1)
            binary = floor( binary - math.pow(10, -i-1) )
        else
            pCity:SetNumRealBuilding(tShipProdDummies[-i-1], 0)
        end
    end
end

function Jar_AmalfiUABonusEveryXTRSlot(iPlayer)
	local pPlayer = Players[iPlayer]
	if (HasTrait(pPlayer, traitAmalfiID) and pPlayer:IsAlive()) then
		--if not pPlayer:GetCapitalCity() then return end
		local capitalcity = pPlayer:GetCapitalCity()
		capitalcity:SetNumRealBuilding(DummyFreePolicy, 0)
		local numTRslots = pPlayer:GetNumInternationalTradeRoutesAvailable()
		local numpolicy = math.min(UAFreePolicyCap, math.floor(numTRslots *UAfactor) )
		if numTRslots > 0 then
			TRPolicyCheck = load( pPlayer, "TRPolicyCheck" );
			if TRPolicyCheck == nil then TRPolicyCheck = 0 end
			if numpolicy > TRPolicyCheck then
				capitalcity:SetNumRealBuilding(DummyFreePolicy, 1)
				save( pPlayer, "TRPolicyCheck", TRPolicyCheck+1)
			end
			for pCity in pPlayer:Cities() do
				local NavalProdBonus = math.floor(NavalUnitProdBoost *numTRslots *UAfactor)
				local bnum = math.min( NavalProdBonus, NavalUnitProdBoostMax)
				Jar_AddShipProdDummy(pCity, bnum)
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiUABonusEveryXTRSlot) end;
--==========================================================================================================================
-- UI: SCARO
--========================================================================================================================
function Jar_AmalfiBorgoEffect(iPlayer)
	local pPlayer = Players[iPlayer]
    local iTeam = pPlayer:GetTeam()
    local pTeam = Teams[iTeam]
	if pPlayer:CountAllImprovement(iBorgoCostiero, true) <1 then return end
	if (HasTrait(pPlayer, traitAmalfiID) and pPlayer:IsAlive()) then
		local tTextstring = {}
		for pCity in pPlayer:Cities() do
			local borgonum = 0
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pPlot = pCity:GetCityIndexPlot(i)
				if pPlot and pPlot:GetOwner() == iPlayer and pPlot:GetImprovementType() == iBorgoCostiero then
					borgonum = borgonum +1
					--
					for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
						local pAdjacentPlot = Map.PlotDirection(pPlot:GetX(), pPlot:GetY(), direction)
						if pAdjacentPlot and pAdjacentPlot:IsUnit() and pAdjacentPlot:IsWater() then
							for i = 0, pAdjacentPlot:GetNumUnits() - 1 do
								local zUnit = pAdjacentPlot:GetUnit(i)
								local pOwner = Players[zUnit:GetOwner()]
								local iOwnerTeam = pOwner:GetTeam()
								if pTeam:IsAtWar(iOwnerTeam) or pOwner:IsBarbarian() then
									zUnit:ChangeDamage(BorgoShipDamage)
								end
							end
						end	
					end
					--
				end
			end
			local dummy2add = math.min(borgonum*BorgoTRrange, BorgoTRrangeMax)
			pCity:SetNumRealBuilding(DummyBorgoTRrange, dummy2add)
			local textstr = pCity:GetName() .. ": +" .. dummy2add .. "% [NEWLINE]"
			if dummy2add > 0 then
				table.insert(tTextstring, textstr)
			end
		end	
		-- Notification
		local sTitle = "[ICON_TRADE] TR bonus range from " .. Locale.ConvertTextKey( GameInfo.Improvements[iBorgoCostiero].Description )
		local sText = ""
		local icheck = 0
		for k, v in pairs(tTextstring) do
			sText = sText .. k .. ") " .. v
			icheck = k
		end
		if pPlayer:IsHuman() and icheck>0 then
			pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle)
		end
		for pUnit in pPlayer:Units() do
			local unitplot = pUnit:GetPlot()
			local bcheck = false
			if pUnit:GetDomainType() == DomainTypes.DOMAIN_SEA then				
				bcheck = unitplot:GetImprovementType() == iBorgoCostiero
				if not bcheck then
					print("Ship near Borgo Costiero. bcheck: ", bcheck)
					for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
						local pAdjacentPlot = Map.PlotDirection(unitplot:GetX(), unitplot:GetY(), direction)
						if pAdjacentPlot and pAdjacentPlot:GetImprovementType() == iBorgoCostiero then
							bcheck = true
							break
						end
					end
				end
			end
			pUnit:SetHasPromotion(BorgoSuppliesPromo, bcheck)
			if bcheck then
				pUnit:ChangeDamage(BorgoShipHeal)
			end
			--print("Borgo Effect: ", unitplot:GetX(), unitplot:GetY() , bcheck)
		end
	end
end

if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiBorgoEffect) end;
--==========================================================================================================================
-- UU: COMITE and FONDACO
--========================================================================================================================
function Jar_AmalfiFondacoShadowTR(iPlayer)
	local pPlayer = Players[iPlayer]
	local numfondaci = Jar_CountFondaci()
	if numfondaci<1 then return end
	if not pPlayer:IsAlive() then return end
	if not HasTrait(pPlayer, traitAmalfiID) then return end
	local tTextstring = {}
	for otherID = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local otherPlayer = Players[otherID]
		if Jar_PlayerHasFondaco(otherPlayer) and iPlayer ~= otherID then
			local iGold = 0
			local iScience = 0
			local iCulture = 0
			for _, route in ipairs(pPlayer:GetTradeRoutes()) do
				if route.FromID == iPlayer and route.ToID == otherID then
					if route.FromGPT > iGold then 
						iGold = 	route.FromGPT
						iScience = 	route.FromScience
						iCulture = 	route.FromCulture
					end
				end
			end
			for _, route in ipairs(pPlayer:GetTradeRoutesAvailable()) do
				if route.FromID == iPlayer and route.ToID == otherID then
					if route.FromGPT > iGold then 
						iGold = 	route.FromGPT
						iScience = 	route.FromScience
						iCulture = 	route.FromCulture
					end
				end
			end
			-- give gold
			local goldbonus = math.floor(iGold/100)
			if goldbonus >0 then
				pPlayer:ChangeGold(goldbonus)
			end
			-- give science
			local sciencebonus = iScience
			if sciencebonus >0 then
				local pTeam = Teams[pPlayer:GetTeam()]
				local pTeamTechs = pTeam:GetTeamTechs()
				pTeamTechs:ChangeResearchProgressTimes100(pPlayer:GetCurrentResearch(), sciencebonus)
			end
			-- give culture
			if iCulture>0 and iCulture<100 then iCulture=100 end
			local CultureBonus = math.floor(iCulture/100)
			if CultureBonus >0 then
				pPlayer:ChangeJONSCulture(CultureBonus)
			end
			-------------------------------------------
			local textstr
			local influenceChange=0
			if otherPlayer:IsMinorCiv() then
				local numdraw = GetRandom(1, 100)
				local influencerandomBonus = 0
				if numdraw <=60 then 						-- 1 with 60% probability
					influencerandomBonus = 1
				elseif numdraw >60 and numdraw <=94 then 	-- 0 with 34% probability
					influencerandomBonus = 0
				elseif numdraw >94 and numdraw <=99 then 	-- 2 with 5% probability
					influencerandomBonus = 2
				else										-- 3 with 1% probability
					influencerandomBonus = 3
				end
				influenceChange = CSInfluenceFondacoBase + influencerandomBonus
				otherPlayer:ChangeMinorCivFriendshipWithMajor(pPlayer, influenceChange)
				textstr = Locale.ConvertTextKey( GameInfo.MinorCivilizations[otherPlayer:GetMinorCivType()].Description ) .. "."
			else
				textstr= Locale.ConvertTextKey( GameInfo.Civilizations[otherPlayer:GetCivilizationType()].ShortDescription ) .. "."
			end
			if influenceChange >0 then
				textstr= textstr .. " +" .. influenceChange .. "[ICON_INFLUENCE],"
			end
			if goldbonus >0 then
				textstr= textstr .. " + " .. goldbonus .. " [ICON_GOLD]"
			end
			if sciencebonus >0 then
				textstr= textstr .. ", +" .. sciencebonus/100 .. " [ICON_RESEARCH]"
			end
			if CultureBonus >0 then
				textstr= textstr ..", +" .. CultureBonus .. " [ICON_CULTURE]"
			end
			textstr = textstr .. "[NEWLINE]"
			table.insert(tTextstring, textstr)
		end
	end		
	-- Notification
	local sTitle = "List of Established Fondaci"
	local sText = ""
	local icheck = 0
	for k, v in pairs(tTextstring) do
		sText = sText .. k .. ") " .. v
		icheck = k
	end
	if pPlayer:IsHuman() and icheck>0 then
		pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText, sTitle) --, ElectedPlotX, ElectedPlotY)
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiFondacoShadowTR) end

-- Grant diplomatic bonus from the civ where you have established a Fondaco
local AFondacoDiploModifier = GameInfoTypes.DIPLOMODIFIER_JAR_AMALFI_FONDACO

function Jar_AmalfiFondacoDiploModifier(iDiploModifier, iFromPlayer, iToPlayer)
	local iModifier = 0
	local pToPlayer = Players[iToPlayer]
	local pFromPlayer = Players[iFromPlayer];
	if iDiploModifier == AFondacoDiploModifier then
		if Jar_PlayerHasFondaco(pFromPlayer) then
			iModifier = -5
		end
		--print(string.format("FondacoDiploModifier(%s, %i, %s, %s)", GameInfo.DiploModifiers[iDiploModifier].Type, iModifier, Players[iFromPlayer]:GetCivilizationDescription(), Players[iToPlayer]:GetCivilizationDescription()))
	end
	return iModifier
end
	
GameEvents.GetDiploModifier.Add(Jar_AmalfiFondacoDiploModifier)
--

function Jar_CanEstablishFondaco(unit) -- checks if a unit is on a valid plot
	local pBool = false;
	local pPlot = unit:GetPlot();
	if unit:MovesLeft() <= 1 then return pBool end
	if pPlot ~= nil then
		local iOwner = pPlot:GetOwner()
		local pOwner = Players[iOwner]
		if (not pPlot:IsCity()) then
			if iOwner ~= -1 and not pOwner:IsBarbarian() then
				if Jar_PlayerHasFondaco(pOwner) then return pBool end
				if pPlot:GetTerrainType() == iCoast or pPlot:GetTerrainType() == iOcean then return pBool end
				if unit:GetOwner() ~= pPlot:GetOwner() then
					pBool = true;
				end
			end
		end
	end
	return pBool
end

function Jar_AddTRslotDummy(pCity, integer)
	print("Jar_AddTRslotDummy: " .. pCity:GetName() .. "; ".. integer)
	local binary = basen(integer,2)
    local numcifre = string.len(tostring(binary))
    for i=-numcifre,-1 do
        if string.len(tostring(binary)) == -i and binary~=0 then
            pCity:SetNumRealBuilding(tTRslotDummies[-i-1], 1)
            binary = floor( binary - math.pow(10, -i-1) )
        else
            pCity:SetNumRealBuilding(tTRslotDummies[-i-1], 0)
        end
    end
end

function Jar_EstablishFondaco(pPlayer, pUnit)
	local pUnitPlot = pUnit:GetPlot();
	local otherplayerID = pUnitPlot:GetOwner()
	local pOtherPlayer = Players[otherplayerID]
	local pCity = Neirai_GetNearestCity(pOtherPlayer, pUnitPlot)
	if not pUnitPlot:GetPlotCity() then
		pUnitPlot:SetImprovementType(-1)
		pUnitPlot:SetImprovementType(iFondaco)
	end
	pUnit:Kill()	
	local textstrF = ""
	if pOtherPlayer:IsMinorCiv() then
		textstrF = Locale.ConvertTextKey( GameInfo.MinorCivilizations[pOtherPlayer:GetMinorCivType()].Description )
	else
		textstrF= Locale.ConvertTextKey( GameInfo.Civilizations[pOtherPlayer:GetCivilizationType()].ShortDescription )
	end
	print("Amalfi established a fondaco in the city of ".. pCity:GetName() .. " (" .. textstrF .. ").")
	
	local numfondaci = Jar_CountFondaci()
	local fondacibefore = math.floor( numfondaci/AddTRfromXFondaci)	
	pCity:SetNumRealBuilding(DummyFondaco, 1);
	numfondaci = numfondaci +1
	local fondaciafter  = math.floor( numfondaci/AddTRfromXFondaci)
	print("Num of established Fondaci: " .. numfondaci)
	local pCapital = pPlayer:GetCapitalCity()
	numTRSlots2Add =  load( pPlayer, "NumFondaciTR" );
	if numTRSlots2Add == nil then numTRSlots2Add = 0 end
	if fondaciafter > fondacibefore then
		Jar_AddTRslotDummy(pCapital, numTRSlots2Add+1)
		save( pPlayer, "NumFondaciTR", numTRSlots2Add+1)		
		if pPlayer:IsHuman() then
			Events.GameplayAlertMessage(Locale.ConvertTextKey("[COLOR_POSITIVE_TEXT]Establishing a Fondaco provided an additional Trade Route slot for Amalfi.[ENDCOLOR]" ))
		end
		print("Amalfi UU: +1 Trade Route slot from number of Fondaci (" .. numfondaci .. ").")
	end	
end

function Jar_AIEstablishesFondaco(iUnitOwner, iUnit, iUnitType, iPlotX, iPlotY)
	local aPlayer = Players[iUnitOwner]
	if aPlayer:IsHuman() then return end
	if not aPlayer:IsAlive() then return end
	if iUnitType == iComite then
		local numfondaci = Jar_CountFondaci()
		local fondacibefore = math.floor( numfondaci/AddTRfromXFondaci)
		local tFondacoList = {}
		local tFondacoDist = {}
		for playerID = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
			local pPlayer = Players[playerID];
			if pPlayer:IsAlive() and not Jar_PlayerHasFondaco(pPlayer) then
				table.insert(tFondacoList, playerID);				
				local pCityPlot = pPlayer:GetCapitalCity():Plot()
				local distance = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), iPlotX, iPlotY)
				tFondacoDist[playerID]= distance
			end
		end
		local chosenID
		if (#tFondacoList > 0) then
			local iDistance = 9999
			for k, v in pairs(tFondacoDist) do
				if v < iDistance then
					iDistance = v
					chosenID = k
				end
			end
		end
		local cPlayer = Players[chosenID]
		local cCapital = cPlayer:GetCapitalCity()
		cCapital:SetNumRealBuilding(DummyFondaco, 1);
		for i = 0, cCapital:GetNumCityPlots() - 1, 1 do
			local pPlot = cCapital:GetCityIndexPlot(i)
			if pPlot:GetResourceType() == -1 and pPlot:GetImprovementType() == -1 and pPlot:GetOwner() == chosenID and pPlot:GetFeatureType() == -1 and not pPlot:IsWater() then
				pPlot:SetImprovementType(iFondaco)							
				if cPlayer:IsHuman() then		
					-- Notification
					local sTitle = "Amalfitan Merchant"
					local sText = "Amalfi established a Fondaco in [ICON_CAPITAL] " .. cCapital:GetName()
					cPlayer:AddNotification(NotificationTypes.NOTIFICATION_CITY_TILE, sText, sTitle, pPlot:GetX(), pPlot:GetY())
					print("AI Amalfi established a Fondaco in Human player's Capital.")
				end
				break
			end
		end
		numfondaci = numfondaci +1
		local fondaciafter  = math.floor( numfondaci/AddTRfromXFondaci)
		local aCapital = aPlayer:GetCapitalCity()
		numTRSlots2Add =  load( aPlayer, "NumFondaciTR" );
		if numTRSlots2Add == nil then numTRSlots2Add = 0 end
		if fondaciafter > fondacibefore then
			Jar_AddTRslotDummy(aCapital, numTRSlots2Add+1)
			save( aPlayer, "NumFondaciTR", numTRSlots2Add+1)
		end
	end
end
if g_IsTraitActive then GameEvents.UnitCreated.Add(Jar_AIEstablishesFondaco) end

function Jar_AmalfiCapitalConquered(oldOwnerID, capital, plotX, plotY, newOwnerID)
	local city = Map.GetPlot(plotX, plotY):GetPlotCity()
	local pConqueror = Players[newOwnerID]
	local pConquered = Players[oldOwnerID]
	-- city with TR slot dummies is lost (capital)
	if HasTrait(pConquered, traitAmalfiID) and pConquered:IsAlive() then
		numTRSlots2Add =  load( pConquered, "NumFondaciTR" );
		if numTRSlots2Add == nil then numTRSlots2Add = 0 end
		if numTRSlots2Add > 0 then
			local bcheck = true
			for power=0,maxpower2 do
				if pConquered:CountNumBuildings(tTRslotDummies[power]) > 0 then
					bcheck = false
					break
				end
			end
			if bcheck then
				local newCapital = pConquered:GetCapitalCity()
				Jar_AddTRslotDummy(newCapital, numTRSlots2Add)
			end
		end
	-- capital is reconquered
	elseif city:IsOriginalCapital() and pConqueror == Players[city:GetOriginalOwner()] and HasTrait(pConqueror, traitAmalfiID) then
		numTRSlots2Add =  load( pConqueror, "NumFondaciTR" );
		if numTRSlots2Add == nil then numTRSlots2Add = 0 end
		if numTRSlots2Add > 0 then
			local newCapital = pConqueror:GetCapitalCity()
			Jar_AddTRslotDummy(newCapital, numTRSlots2Add)
			for pCity in pConqueror:Cities() do
				if not pCity:IsCapital() then
					for power=0,maxpower2 do
						pCity:SetNumRealBuilding(tTRslotDummies[power], 0)
					end
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.CityCaptureComplete.Add(Jar_AmalfiCapitalConquered) end
--==========================================================================================================================
-- CBP MISSION EVENTS COMPATIBILITY
--==========================================================================================================================
-- Jar_Comite_CustomMissionStart
local function Jar_Comite_CustomMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionComiteBuildFondaco then
		Jar_EstablishFondaco(player, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

-- Jar_Comite_CustomMissionCompleted
local function Jar_Comite_CustomMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionComiteBuildFondaco)
end

-- Jar_Comite_CustomMissionPossible
local function Jar_Comite_CustomMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	if missionID == missionComiteBuildFondaco then
		if unit:GetUnitType() == iComite then
			return Jar_CanEstablishFondaco(unit)
		end
	end	
	return false
end

GameEvents.CustomMissionStart.Add(Jar_Comite_CustomMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_Comite_CustomMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_Comite_CustomMissionPossible)
--==========================================================================================================================
-- More Unique Components for Vox Populi
local iSaettia	 = GameInfoTypes["UNIT_JAR_SAETTIA"]
local MPSaettia	 = 1
local iCartiera  = GameInfoTypes["BUILDING_JAR_CARTIERA"]
local DummyPaper = GameInfoTypes["BUILDING_JAR_DUMMY_PAPER"]

-- UU2: Saettia
function Jar_AmalfiSaettiaBoarding(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:GetNumUnitPromotions(GameInfoTypes.PROMOTION_JAR_FASTCARGO) <1 then return end
	if not pPlayer:IsAlive() then return end
	local pCapital = pPlayer:GetCapitalCity()
	for pUnit in pPlayer:Units() do
		if pUnit:GetDomainType() == DomainTypes.DOMAIN_LAND then
			local unitplot = pUnit:GetPlot();
			local bcheck = false
			for i = 0, unitplot:GetNumUnits() - 1 do
				local wUnit = unitplot:GetUnit(i)
				local iOwner = wUnit:GetOwner()
				if iOwner == playerID and wUnit:GetUnitType() == iSaettia then
					pUnit:SetMoves(wUnit:GetMoves())
					pCapital:ChangeSpecialistGreatPersonProgressTimes100(GameInfoTypes["SPECIALIST_MERCHANT"], MPSaettia*100)
					if pPlayer:IsHuman() then
						Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(unitplot:GetX(), unitplot:GetY()))), "[COLOR_YELLOW]+" .. MPSaettia .." [ICON_GREAT_MERCHANT][ENDCOLOR]", 2)
					end
					bcheck = true
					break
				end
			end			
			pUnit:SetHasPromotion(GameInfoTypes.PROMOTION_JAR_FASTCARGO, bcheck)
		elseif pUnit:GetUnitType() == iSaettia then
			local unitplot = pUnit:GetPlot();
			local mcs = GameInfo.Units["UNIT_JAR_SAETTIA"].Combat
			local rcs = GameInfo.Units["UNIT_JAR_SAETTIA"].RangedCombat
			local bcheck = false
			for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
				local pAdjacentPlot = Map.PlotDirection(unitplot:GetX(), unitplot:GetY(), direction)
				if pAdjacentPlot and Jar_PlotHasTradeRoute(pAdjacentPlot, pPlayer) then
					bcheck = true
					break
				end
			end
			if (not bcheck) and Jar_PlotHasTradeRoute(unitplot, pPlayer) then
				bcheck = true
			end
			if bcheck then
				mcs = mcs +3
				rcs = rcs +3
			end
			pUnit:SetBaseCombatStrength(mcs)
			pUnit:SetBaseRangedCombatStrength(rcs)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiSaettiaBoarding) end

-- UB: Cartiera/Paper mill
function Jar_AmalfiCartieraGivePaper(playerID)
	local pPlayer = Players[playerID]
	local pCapital = pPlayer:GetCapitalCity()
	if HasTrait(pPlayer, traitAmalfiID) and pPlayer:IsAlive() and pCapital then
		local countCartiera = pPlayer:CountNumBuildings(iCartiera)
		pCapital:SetNumRealBuilding(DummyPaper, 1+ math.floor(countCartiera/3));			
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_AmalfiCartieraGivePaper) end
--=======================================================================================================================
--TROLLER'S ADDITIONAL ACHIEVEMENTS
--=======================================================================================================================
local TAAid = "432bc547-eb05-4189-9e46-232dbde8f09f"
local isAAActive = Game_IsUsingMod(TAAid)
local iCargoShip = GameInfoTypes["UNIT_CARGO_SHIP"]
----------------------------------------
function UnlockAmalfiAchievement(iPlayer)
	local pPlayer = Players[iPlayer]
	if IsAAUnlocked('AA_JAR_AMALFI_SPECIAL') then return end
	if (HasTrait(pPlayer, traitAmalfiID) and pPlayer:IsAlive()) then
		local cargonum = 0
		for pUnit in pPlayer:Units() do
			if pUnit:GetUnitType() == iCargoShip then
				cargonum = cargonum +1
			end
		end
		if cargonum >=12 then
			UnlockAA('AA_JAR_AMALFI_SPECIAL')
		end
	end
end
if isAAActive then
	GameEvents.PlayerDoTurn.Add(UnlockAmalfiAchievement)
end
---------------------------------------------------------
print("Amalfi functions loaded")
--==========================================================================================================================
--==========================================================================================================================